@@ -20,7 +20,7 @@ from typing import Optional, Union
 import torch
 
 from .jit import FLASHINFER_CSRC_DIR, has_prebuilt_ops, load_cuda_ops
-from .utils import get_cuda_stream, register_custom_op, register_fake_op
+from .utils import register_custom_op, register_fake_op
 
 _sampling_module = None
 
@@ -51,19 +51,18 @@ def get_sampling_module():
             deterministic: bool,
             generator: Optional[torch.Generator],
         ) -> torch.Tensor:
-            with probs.device as device:
-                probs = probs.float()
-                batch_size = indices.size(0) if indices is not None else probs.size(0)
-                samples = torch.empty(batch_size, dtype=torch.int32, device=device)
-                module.sampling_from_probs(
-                    probs,
-                    samples,
-                    indices,
-                    deterministic,
-                    generator,
-                    get_cuda_stream(device),
-                )
-                return samples
+            device = probs.device
+            probs = probs.float()
+            batch_size = indices.size(0) if indices is not None else probs.size(0)
+            samples = torch.empty(batch_size, dtype=torch.int32, device=device)
+            module.sampling_from_probs.default(
+                probs,
+                samples,
+                indices,
+                deterministic,
+                generator,
+            )
+            return samples
 
         @register_fake_op("flashinfer::sampling_from_probs")
         def _fake_sampling_from_probs(
@@ -86,24 +85,23 @@ def get_sampling_module():
             deterministic: bool,
             generator: Optional[torch.Generator],
         ) -> torch.Tensor:
-            with probs.device as device:
-                probs = probs.float()
-                maybe_top_p_arr = (
-                    maybe_top_p_arr.float() if maybe_top_p_arr is not None else None
-                )
-                batch_size = indices.size(0) if indices is not None else probs.size(0)
-                samples = torch.empty(batch_size, dtype=torch.int32, device=device)
-                module.top_p_sampling_from_probs(
-                    probs,
-                    samples,
-                    indices,
-                    maybe_top_p_arr,
-                    top_p_val,
-                    deterministic,
-                    generator,
-                    get_cuda_stream(device),
-                )
-                return samples
+            device = probs.device
+            probs = probs.float()
+            maybe_top_p_arr = (
+                maybe_top_p_arr.float() if maybe_top_p_arr is not None else None
+            )
+            batch_size = indices.size(0) if indices is not None else probs.size(0)
+            samples = torch.empty(batch_size, dtype=torch.int32, device=device)
+            module.top_p_sampling_from_probs.default(
+                probs,
+                samples,
+                indices,
+                maybe_top_p_arr,
+                top_p_val,
+                deterministic,
+                generator,
+            )
+            return samples
 
         @register_fake_op("flashinfer::top_p_sampling_from_probs")
         def _fake_top_p_sampling_from_probs(
@@ -128,24 +126,23 @@ def get_sampling_module():
             deterministic: bool,
             generator: Optional[torch.Generator],
         ) -> torch.Tensor:
-            with probs.device as device:
-                probs = probs.float()
-                batch_size = indices.size(0) if indices is not None else probs.size(0)
-                maybe_top_k_arr = (
-                    maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
-                )
-                samples = torch.empty(batch_size, dtype=torch.int32, device=device)
-                module.top_k_sampling_from_probs(
-                    probs,
-                    samples,
-                    indices,
-                    maybe_top_k_arr,
-                    top_k_val,
-                    deterministic,
-                    generator,
-                    get_cuda_stream(device),
-                )
-                return samples
+            device = probs.device
+            probs = probs.float()
+            batch_size = indices.size(0) if indices is not None else probs.size(0)
+            maybe_top_k_arr = (
+                maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
+            )
+            samples = torch.empty(batch_size, dtype=torch.int32, device=device)
+            module.top_k_sampling_from_probs.default(
+                probs,
+                samples,
+                indices,
+                maybe_top_k_arr,
+                top_k_val,
+                deterministic,
+                generator,
+            )
+            return samples
 
         @register_fake_op("flashinfer::top_k_sampling_from_probs")
         def _fake_top_k_sampling_from_probs(
@@ -171,24 +168,23 @@ def get_sampling_module():
             deterministic: bool,
             generator: Optional[torch.Generator],
         ) -> torch.Tensor:
-            with probs.device as device:
-                probs = probs.float()
-                maybe_min_p_arr = (
-                    maybe_min_p_arr.float() if maybe_min_p_arr is not None else None
-                )
-                batch_size = indices.size(0) if indices is not None else probs.size(0)
-                samples = torch.empty(batch_size, dtype=torch.int32, device=device)
-                module.min_p_sampling_from_probs(
-                    probs,
-                    samples,
-                    indices,
-                    maybe_min_p_arr,
-                    min_p_val,
-                    deterministic,
-                    generator,
-                    get_cuda_stream(device),
-                )
-                return samples
+            device = probs.device
+            probs = probs.float()
+            maybe_min_p_arr = (
+                maybe_min_p_arr.float() if maybe_min_p_arr is not None else None
+            )
+            batch_size = indices.size(0) if indices is not None else probs.size(0)
+            samples = torch.empty(batch_size, dtype=torch.int32, device=device)
+            module.min_p_sampling_from_probs.default(
+                probs,
+                samples,
+                indices,
+                maybe_min_p_arr,
+                min_p_val,
+                deterministic,
+                generator,
+            )
+            return samples
 
         # torch library for top_k_top_p_sampling_from_probs
 
@@ -205,29 +201,28 @@ def get_sampling_module():
             deterministic: bool,
             generator: Optional[torch.Generator],
         ) -> torch.Tensor:
-            with probs.device as device:
-                probs = probs.float()
-                maybe_top_k_arr = (
-                    maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
-                )
-                maybe_top_p_arr = (
-                    maybe_top_p_arr.float() if maybe_top_p_arr is not None else None
-                )
-                batch_size = indices.size(0) if indices is not None else probs.size(0)
-                samples = torch.empty(batch_size, dtype=torch.int32, device=device)
-                module.top_k_top_p_sampling_from_probs(
-                    probs,
-                    samples,
-                    indices,
-                    maybe_top_k_arr,
-                    top_k_val,
-                    maybe_top_p_arr,
-                    top_p_val,
-                    deterministic,
-                    generator,
-                    get_cuda_stream(device),
-                )
-                return samples
+            device = probs.device
+            probs = probs.float()
+            maybe_top_k_arr = (
+                maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
+            )
+            maybe_top_p_arr = (
+                maybe_top_p_arr.float() if maybe_top_p_arr is not None else None
+            )
+            batch_size = indices.size(0) if indices is not None else probs.size(0)
+            samples = torch.empty(batch_size, dtype=torch.int32, device=device)
+            module.top_k_top_p_sampling_from_probs.default(
+                probs,
+                samples,
+                indices,
+                maybe_top_k_arr,
+                top_k_val,
+                maybe_top_p_arr,
+                top_p_val,
+                deterministic,
+                generator,
+            )
+            return samples
 
         @register_fake_op("flashinfer::top_k_top_p_sampling_from_probs")
         def _fake_top_k_top_p_sampling_from_probs(
@@ -252,20 +247,19 @@ def get_sampling_module():
             maybe_top_p_arr: Optional[torch.Tensor],
             top_p_val: float,
         ) -> torch.Tensor:
-            with probs.device as device:
-                probs = probs.float()
-                maybe_top_p_arr = (
-                    maybe_top_p_arr.float() if maybe_top_p_arr is not None else None
-                )
-                renorm_probs = torch.empty_like(probs)
-                module.top_p_renorm_probs(
-                    probs,
-                    renorm_probs,
-                    maybe_top_p_arr,
-                    top_p_val,
-                    get_cuda_stream(device),
-                )
-                return renorm_probs
+            device = probs.device
+            probs = probs.float()
+            maybe_top_p_arr = (
+                maybe_top_p_arr.float() if maybe_top_p_arr is not None else None
+            )
+            renorm_probs = torch.empty_like(probs)
+            module.top_p_renorm_probs.default(
+                probs,
+                renorm_probs,
+                maybe_top_p_arr,
+                top_p_val,
+            )
+            return renorm_probs
 
         @register_fake_op("flashinfer::top_p_renorm_probs")
         def _fake_top_p_renorm_probs(
@@ -283,20 +277,19 @@ def get_sampling_module():
             maybe_top_k_arr: Optional[torch.Tensor],
             top_k_val: int,
         ) -> torch.Tensor:
-            with probs.device as device:
-                probs = probs.float()
-                maybe_top_k_arr = (
-                    maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
-                )
-                renorm_probs = torch.empty_like(probs)
-                module.top_k_renorm_probs(
-                    probs,
-                    renorm_probs,
-                    maybe_top_k_arr,
-                    top_k_val,
-                    get_cuda_stream(device),
-                )
-                return renorm_probs
+            device = probs.device
+            probs = probs.float()
+            maybe_top_k_arr = (
+                maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
+            )
+            renorm_probs = torch.empty_like(probs)
+            module.top_k_renorm_probs.default(
+                probs,
+                renorm_probs,
+                maybe_top_k_arr,
+                top_k_val,
+            )
+            return renorm_probs
 
         @register_fake_op("flashinfer::top_k_renorm_probs")
         def _fake_top_k_renorm_probs(
@@ -314,20 +307,19 @@ def get_sampling_module():
             maybe_top_k_arr: Optional[torch.Tensor],
             top_k_val: int,
         ) -> torch.Tensor:
-            with logits.device as device:
-                logits = logits.float()
-                maybe_top_k_arr = (
-                    maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
-                )
-                mask_logits = torch.empty_like(logits)
-                module.top_k_mask_logits(
-                    logits,
-                    mask_logits,
-                    maybe_top_k_arr,
-                    top_k_val,
-                    get_cuda_stream(device),
-                )
-                return mask_logits
+            device = logits.device
+            logits = logits.float()
+            maybe_top_k_arr = (
+                maybe_top_k_arr.int() if maybe_top_k_arr is not None else None
+            )
+            mask_logits = torch.empty_like(logits)
+            module.top_k_mask_logits.default(
+                logits,
+                mask_logits,
+                maybe_top_k_arr,
+                top_k_val,
+            )
+            return mask_logits
 
         @register_fake_op("flashinfer::top_k_mask_logits")
         def _fake_top_k_mask_logits(
@@ -352,28 +344,25 @@ def get_sampling_module():
             deterministic: bool,
             generator: Optional[torch.Generator],
         ) -> torch.Tensor:
-            with draft_probs.device as device:
-                draft_probs = draft_probs.float()
-                draft_token_ids = draft_token_ids.int()
-                target_probs = target_probs.float()
-                output_accepted_token_num = output_accepted_token_num.int()
-                output_emitted_token_num = output_emitted_token_num.int()
-                b, n = draft_token_ids.shape
-                output_token_ids = torch.empty(
-                    (b, n + 1), dtype=torch.int32, device=device
-                )
-                module.chain_speculative_sampling(
-                    draft_probs,
-                    draft_token_ids,
-                    target_probs,
-                    output_token_ids,
-                    output_accepted_token_num,
-                    output_emitted_token_num,
-                    deterministic,
-                    generator,
-                    get_cuda_stream(device),
-                )
-                return output_token_ids
+            device = draft_probs.device
+            draft_probs = draft_probs.float()
+            draft_token_ids = draft_token_ids.int()
+            target_probs = target_probs.float()
+            output_accepted_token_num = output_accepted_token_num.int()
+            output_emitted_token_num = output_emitted_token_num.int()
+            b, n = draft_token_ids.shape
+            output_token_ids = torch.empty((b, n + 1), dtype=torch.int32, device=device)
+            module.chain_speculative_sampling.default(
+                draft_probs,
+                draft_token_ids,
+                target_probs,
+                output_token_ids,
+                output_accepted_token_num,
+                output_emitted_token_num,
+                deterministic,
+                generator,
+            )
+            return output_token_ids
 
         @register_fake_op("flashinfer::chain_speculative_sampling")
         def _fake_chain_speculative_sampling(
@@ -486,6 +475,7 @@ def top_p_sampling_from_probs(
 ) -> torch.Tensor:
     r"""Fused GPU kernel for top-p sampling (nucleus sampling) from probabilities,
     this operator implements GPU-based rejection sampling without explicit sorting.
+    Check the `blog post <https://flashinfer.ai/2025/03/10/sampling.html>`_ for more details.
 
     The multiple rounds of rejection sampling are implemented in a single CUDA kernel,
     which is more efficient than the naive implementation that launches a series of kernels.
@@ -567,6 +557,7 @@ def top_k_sampling_from_probs(
 ) -> torch.Tensor:
     r"""Fused GPU kernel for top-k sampling from probabilities,
     this operator implements GPU-based rejection sampling without explicit sorting.
+    Check the `blog post <https://flashinfer.ai/2025/03/10/sampling.html>`_ for more details.
 
     The multiple rounds of rejection sampling are implemented in a single CUDA kernel,
     which is more efficient than the naive implementation that launches a series of kernels.
@@ -649,6 +640,7 @@ def min_p_sampling_from_probs(
     r"""Fused GPU kernel for `min_p sampling <https://arxiv.org/abs/2407.01082>`_ from probabilities,
 
     this operator implements GPU-based rejection sampling without explicit sorting.
+    Check the `blog post <https://flashinfer.ai/2025/03/10/sampling.html>`_ for more details.
 
     The multiple rounds of rejection sampling are implemented in a single CUDA kernel,
     which is more efficient than the naive implementation that launches a series of kernels.
@@ -728,6 +720,7 @@ def top_k_top_p_sampling_from_logits(
     r"""Fused GPU kernel for top-k and top-p sampling from pre-softmax logits,
 
     this operator implements GPU-based rejection sampling without explicit sorting.
+    Check the `blog post <https://flashinfer.ai/2025/03/10/sampling.html>`_ for more details.
 
     The multiple rounds of rejection sampling are implemented in a single CUDA kernel,
     which is more efficient than the naive implementation that launches a series of kernels.
@@ -847,6 +840,7 @@ def top_k_top_p_sampling_from_probs(
     r"""Fused GPU kernel for top-k and top-p sampling from probabilities,
 
     this operator implements GPU-based rejection sampling without explicit sorting.
+    Check the `blog post <https://flashinfer.ai/2025/03/10/sampling.html>`_ for more details.
 
     The multiple rounds of rejection sampling are implemented in a single CUDA kernel,
     which is more efficient than the naive implementation that launches a series of kernels.
@@ -1151,9 +1145,7 @@ def chain_speculative_sampling(
         Shape: ``(batch_size, num_speculate_tokens, vocab_size)``
     draft_token_ids: torch.Tensor
         The draft model's generated token indices.
-        Shape: ``(batch_size, num_specutate_tokens)``
-    target_probs: torch.Tensor
-        Expected to be uniformly distributed in ``[0, 1)``.
+        Shape: ``(batch_size, num_speculate_tokens)``
     target_probs: torch.Tensor
         The probability over vocabulary generated by target model.
         Compared to input :attr:`draft_probs`, the target model's probability has an additional
@@ -1183,7 +1175,7 @@ def chain_speculative_sampling(
         Compared to input :attr:`draft_token_ids`, the output tensor has an additional
         token index at the end for the final token, if all previous tokens are accepted,
         another "bonus" token will be sampled from the target model's probability.
-        Shape: (batch_size, num_specutate_tokens + 1)
+        Shape: (batch_size, num_speculate_tokens + 1)
     output_accepted_token_num: torch.Tensor
         The number of tokens that can be accepted if each token is considered independently for each request.
         This metric does not consider the fact that rejection sampling will stop at the first token that does not
@@ -1207,7 +1199,6 @@ def chain_speculative_sampling(
     >>> # token 2 was sampled from draft model for the first token, and
     >>> # token 1 was sampled from draft model for the second token
     >>> draft_token_ids = torch.tensor([[2, 1]], dtype=torch.int32).to(0)
-    >>> # uniform samples for rejection sampling
     >>> target_probs = torch.tensor([[[0.0, 0.1, 0.6, 0.3], [1.0, 0.0, 0.0, 0.0], [0.7, 0.1, 0.1, 0.1]]]).to(0)
     >>> output_token_ids, output_accepted_token_num, output_accepted_token_num =\
     ...     flashinfer.sampling.chain_speculative_sampling(
