@@ -32,7 +32,7 @@ def warmup_jit():
                     [torch.float16],  # q_dtypes
                     [torch.float16],  # kv_dtypes
                     [64, 128, 256],  # head_dims
-                    [0, 1, 2],  # pos_encoding_modes
+                    [0, 1],  # pos_encoding_modes
                     [False],  # use_sliding_windows
                     [False],  # use_logits_soft_caps
                 )
@@ -40,7 +40,7 @@ def warmup_jit():
                     [torch.float16],  # q_dtypes
                     [torch.float16],  # kv_dtypes
                     [64, 128, 256],  # head_dims
-                    [0, 1, 2],  # pos_encoding_modes
+                    [0, 1],  # pos_encoding_modes
                     [False],  # use_sliding_windows
                     [False],  # use_logits_soft_caps
                     [False],  # use_fp16_qk_reductions
@@ -58,7 +58,7 @@ def warmup_jit():
 @pytest.mark.parametrize("group_size", [1, 4, 8])
 @pytest.mark.parametrize("head_dim", [64, 128, 256])
 @pytest.mark.parametrize("kv_layout", ["HND", "NHD"])
-@pytest.mark.parametrize("pos_encoding_mode", ["NONE", "ROPE_LLAMA", "ALIBI"])
+@pytest.mark.parametrize("pos_encoding_mode", ["NONE", "ROPE_LLAMA"])
 def test_single_decode_tensor_cores(
     kv_len: int,
     num_kv_heads: int,
@@ -68,16 +68,24 @@ def test_single_decode_tensor_cores(
     pos_encoding_mode: str,
 ):
     num_qo_heads = num_kv_heads * group_size
-    q = torch.randn(num_qo_heads, head_dim).to(0).half()
+    q = torch.randn(num_qo_heads, head_dim, device="cuda:0", dtype=torch.float16)
     k = (
-        torch.randn(num_kv_heads, kv_len, head_dim).to(0).half()
+        torch.randn(
+            num_kv_heads, kv_len, head_dim, device="cuda:0", dtype=torch.float16
+        )
         if kv_layout == "HND"
-        else torch.randn(kv_len, num_kv_heads, head_dim).to(0).half()
+        else torch.randn(
+            kv_len, num_kv_heads, head_dim, device="cuda:0", dtype=torch.float16
+        )
     )
     v = (
-        torch.randn(num_kv_heads, kv_len, head_dim).to(0).half()
+        torch.randn(
+            num_kv_heads, kv_len, head_dim, device="cuda:0", dtype=torch.float16
+        )
         if kv_layout == "HND"
-        else torch.randn(kv_len, num_kv_heads, head_dim).to(0).half()
+        else torch.randn(
+            kv_len, num_kv_heads, head_dim, device="cuda:0", dtype=torch.float16
+        )
     )
 
     o = flashinfer.single_decode_with_kv_cache(
@@ -97,7 +105,7 @@ def test_single_decode_tensor_cores(
 @pytest.mark.parametrize("group_size", [1, 4, 8])
 @pytest.mark.parametrize("head_dim", [128, 256])
 @pytest.mark.parametrize("kv_layout", ["HND", "NHD"])
-@pytest.mark.parametrize("pos_encoding_mode", ["NONE", "ROPE_LLAMA", "ALIBI"])
+@pytest.mark.parametrize("pos_encoding_mode", ["NONE", "ROPE_LLAMA"])
 def test_batch_decode_tensor_cores(
     batch_size: int,
     kv_len: int,
@@ -109,22 +117,44 @@ def test_batch_decode_tensor_cores(
     pos_encoding_mode: str,
 ):
     num_qo_heads = num_kv_heads * group_size
-    q = torch.randn(batch_size, num_qo_heads, head_dim).to(0).to(torch.float16)
+    q = torch.randn(
+        batch_size, num_qo_heads, head_dim, device="cuda:0", dtype=torch.float16
+    )
     num_pages_per_seq = (kv_len + page_size - 1) // page_size
     total_num_pages = num_pages_per_seq * batch_size
     kv_data = (
-        torch.randn(total_num_pages, 2, num_kv_heads, page_size, head_dim).to(0) / 10
-        if kv_layout == "HND"
-        else torch.randn(total_num_pages, 2, page_size, num_kv_heads, head_dim).to(0)
+        torch.randn(
+            total_num_pages,
+            2,
+            num_kv_heads,
+            page_size,
+            head_dim,
+            device="cuda:0",
+            dtype=torch.float16,
+        )
         / 10
-    ).to(torch.float16)
-    kv_indptr = torch.arange(0, batch_size + 1).to(0).int() * num_pages_per_seq
-    kv_indices = torch.arange(0, total_num_pages).to(0).int()
+        if kv_layout == "HND"
+        else torch.randn(
+            total_num_pages,
+            2,
+            page_size,
+            num_kv_heads,
+            head_dim,
+            device="cuda:0",
+            dtype=torch.float16,
+        )
+        / 10
+    )
+    kv_indptr = (
+        torch.arange(0, batch_size + 1, device="cuda:0", dtype=torch.int32)
+        * num_pages_per_seq
+    )
+    kv_indices = torch.arange(0, total_num_pages, device="cuda:0", dtype=torch.int32)
     kv_last_page_len = torch.full(
-        (batch_size,), (kv_len - 1) % page_size + 1, dtype=torch.int32
-    ).to(0)
+        (batch_size,), (kv_len - 1) % page_size + 1, dtype=torch.int32, device="cuda:0"
+    )
 
-    workspace_buffer = torch.empty(128 * 1024 * 1024, dtype=torch.int8).to(0)
+    workspace_buffer = torch.empty(128 * 1024 * 1024, dtype=torch.int8, device="cuda:0")
     wrapper = flashinfer.BatchDecodeWithPagedKVCacheWrapper(workspace_buffer, kv_layout)
     wrapper.plan(
         kv_indptr,
@@ -167,7 +197,7 @@ def test_batch_decode_tensor_cores(
 @pytest.mark.parametrize("group_size", [1, 4, 8])
 @pytest.mark.parametrize("head_dim", [128, 256])
 @pytest.mark.parametrize("kv_layout", ["HND", "NHD"])
-@pytest.mark.parametrize("pos_encoding_mode", ["NONE", "ROPE_LLAMA", "ALIBI"])
+@pytest.mark.parametrize("pos_encoding_mode", ["NONE", "ROPE_LLAMA"])
 def test_batch_decode_tensor_cores_cuda_graph(
     batch_size: int,
     kv_len: int,
@@ -179,22 +209,44 @@ def test_batch_decode_tensor_cores_cuda_graph(
     pos_encoding_mode: str,
 ):
     num_qo_heads = num_kv_heads * group_size
-    q = torch.randn(batch_size, num_qo_heads, head_dim).to(0).to(torch.float16)
+    q = torch.randn(
+        batch_size, num_qo_heads, head_dim, device="cuda:0", dtype=torch.float16
+    )
     num_pages_per_seq = (kv_len + page_size - 1) // page_size
     total_num_pages = num_pages_per_seq * batch_size
     kv_data = (
-        torch.randn(total_num_pages, 2, num_kv_heads, page_size, head_dim).to(0) / 10
-        if kv_layout == "HND"
-        else torch.randn(total_num_pages, 2, page_size, num_kv_heads, head_dim).to(0)
+        torch.randn(
+            total_num_pages,
+            2,
+            num_kv_heads,
+            page_size,
+            head_dim,
+            device="cuda:0",
+            dtype=torch.float16,
+        )
         / 10
-    ).to(torch.float16)
-    kv_indptr = torch.arange(0, batch_size + 1).to(0).int() * num_pages_per_seq
-    kv_indices = torch.arange(0, total_num_pages).to(0).int()
+        if kv_layout == "HND"
+        else torch.randn(
+            total_num_pages,
+            2,
+            page_size,
+            num_kv_heads,
+            head_dim,
+            device="cuda:0",
+            dtype=torch.float16,
+        )
+        / 10
+    )
+    kv_indptr = (
+        torch.arange(0, batch_size + 1, device="cuda:0", dtype=torch.int32)
+        * num_pages_per_seq
+    )
+    kv_indices = torch.arange(0, total_num_pages, device="cuda:0", dtype=torch.int32)
     kv_last_page_len = torch.full(
-        (batch_size,), (kv_len - 1) % page_size + 1, dtype=torch.int32
-    ).to(0)
+        (batch_size,), (kv_len - 1) % page_size + 1, dtype=torch.int32, device="cuda:0"
+    )
 
-    workspace_buffer = torch.empty(128 * 1024 * 1024, dtype=torch.int8).to(0)
+    workspace_buffer = torch.empty(128 * 1024 * 1024, dtype=torch.int8, device="cuda:0")
 
     # cuda cores wrapper
     wrapper = flashinfer.BatchDecodeWithPagedKVCacheWrapper(
